/**
 * Copyright (C) 2011 dinstone <dinstone@163.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.dinstone.ut.faststub.internal;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import net.sf.cglib.proxy.Enhancer;
import net.sf.cglib.proxy.InvocationHandler;

import com.dinstone.ut.faststub.ClassUtil;
import com.dinstone.ut.faststub.MethodInterceptor;

/**
 * @author dinstone
 * 
 */
public class ProxyFactory {

    public <T> T createProxyInstance(final Class<T> toStub, MethodInterceptor interceptor) {
        if (toStub.isInterface()) {
            return jdkProxy(toStub, interceptor);
        } else {
            return cglibProxy(toStub, interceptor);
        }
    }

    @SuppressWarnings("unchecked")
    private <T> T jdkProxy(final Class<T> toStub, MethodInterceptor interceptor) {
        ProxyInvocationHandler pih = new ProxyInvocationHandler(toStub, interceptor);
        return (T) Proxy.newProxyInstance(ClassUtil.getClassLoader(), new Class[] { toStub }, pih);
    }

    @SuppressWarnings("unchecked")
    private <T> T cglibProxy(final Class<T> toStub, MethodInterceptor interceptor) {
        final ProxyInvocationHandler pih = new ProxyInvocationHandler(toStub, interceptor);
        return (T) Enhancer.create(toStub, new InvocationHandler() {

            public Object invoke(Object o, Method method, Object[] objects) throws Throwable {
                return pih.invoke(o, method, objects);
            }
        });
    }
}
